\name{allocation}
\alias{allocation}

\title{Definition of the inclusion probabilities}

\description{The function returns the inclusion probabilities for each profile in the input data frame namedata. A profile is given by the values of the YPRED variables and MARG variables.
Expected Coefficient of Variation (CV) and statistics on the convergence of the algorithm are returned as well.
}
\usage{
allocation(param1, output_mw, maxiter = 5, minouter, maxouter, minprof=0,
           integ.solution="NO ",convergence = 0.01)
}

\arguments{
  \item{param1}{LIST returned by parameters function}
  \item{output_mw}{LIST returned by allocation function in the previous sessions. Not required if the first session in launched. 
 A session is carried out when the user launches the allocation function and a partial solution has been obtained (no convergence)}
  \item{maxiter}{maximum number of inner loop iterations within an outerloop}
  \item{minouter}{an integer identifying the index of the first outerloop of the session}
  \item{maxouter}{an integer identifying the index of the last outerloop of the session}
  \item{minprof}{an integer specifying the minimum number of units in the profile (record); by default, its value is 0}
  \item{integ.solution}{character specifying if the number of units in the profile (record) must be an integer; by default, its value is "NO ", if integer "YES"}
  \item{convergence}{a real number defining the convergence threshold of the algorithm}
}
\details{The allocation function is applied after the parameters have been returned by the parameters function. 
The user can apply the allocation function in different sessions. 
At the end of a session an output (output_mw) is returned. This output saves the data of convergence process 
implemented in the previous sessions 
and it becomes the input for a further session. In this way the allocation process can be
 implemented by several steps.
The session management depends on the values of minouter and maxouter according to the following rules: 

(i) first session: minouter must be equal to 100; maxouter must be greater or equal to 100. If maxouter=100 
then only one outer loop is performed;
(ii) following sessions: minouter must be equal to maxouter+1 fixed in the previous session. Example: in the previous session minouter=103 and maxouter=105 then 
in the new session minouter must be equal to 106. If  maxouter is equal to minouter only one outer loop is performed.
  
Depending on the dimension of the allocation problem (number of profiles, 
number of driving variables and number of margins), the function could be time spending.
}

\value{The function returns a LIST of data frames and matrices storing the inclusion probabilities computed at the end of each outer loop
and the information of the convergence process. In the LIST the errors and hist_constraints data frames include the instrumental CV thresholds 
used in the  inner loops and in the last outer loops (see note). 

  \item{output_mw}{data frame containing for each profile the following variables:} 

  \itemize{
  \item{id: }{see parameters function;}
  \item{MARG(I): }{variable identifying the Ith domain partition (margin). See MARG in parameters function;}
  \item{pop: }{see pop in parameters function;}
  \item{size_out100: }{initial profile sample size. See start_size in parameters function;} 
  \item{size_out101: }{profile sample size after the first outerloop;}   
  \item{INC.PROB101: }{Inverse of size_out101. When the record in the namedata data frame is a population unit the value is the inclusion probability after the first outer loop}
  \item{\dots}{\dots}     
  \item{size_outL: }{profile sample size after the Lth outer loop;}   
  \item{INC.PROBL: }{Inverse of size_outL. When the record in the namedata data frame is a population unit the value is the inclusion probability after Lth outer loop.}
  }
 
  \item{errors}{data frame containing the instrumental CV thresholds used in the last inner loop of the last outerloop performed by the function;}

  \item{hist_constraints}{matrix containing the instrumental CV thresholds used in each inner loop by outerloop by the function. 
        The sample variable returnes the sample sizes;}

  \item{hist_expected_cv}{matrix containing information for each inner loop in a given outer loop:}
  \itemize{
  \item{out_iter: }{outer loop indicator variable;}
  \item{in_iter: }{inner loop indicator variable;} 
  \item{sample: }{sample size returned in the given inner loop;}
  \item{cv_eta(I): }{expected CV for Ith combination between driving variables by domain. Note that these expected CVs hold when the algorithm converges 
   to an optimal solution. Otherwise they are theoretical values useful for understanding the trend of convergence. Each CV has the following generic name cv_eta.v.m:d, where: v indicates the variables;
m indicates the partition (margin); d indicates the domain of the given partition};
  }

  \item{hist_stat}{matrix containing information on the convergence process. See hist_expected_cv for the variables 
   out_iter, in_iter and sample. The remaing variables are:}
  \itemize{
  \item{difsample: }{sample size difference between the current and the previous iteration;}
  \item{n_pos: }{number of domains of estimate where the relative difference between the current CV and the CV threshold is positive;}
  \item{n_pos1: }{number of domains of estimate where the relative difference between the current CV and the CV threshold is larger than 1\%;}
  \item{mean_difpos: }{mean of the positive relative differences between the current CVs and the CV thresholds;}
  \item{median_difpos: }{median of the positive relative differences between the current CVs and the CV thresholds;}
  \item{q3_difpos: }{3th quartile of the positive relative differences between the current CVs and the CV thresholds;}
  \item{max_difpos: }{max of the positive relative differences between the current CVs and the CV thresholds.}
  }
  
  \item{convergence}{matrix containing convergence parameters:} 
  \itemize{
  \item{maxiter: }{max number of the inner loop iteration in a outer loop;}
  \item{convergence: }{convergence threshold. The value is compared with difsample size. If difsample<convergence in two contiguous inner loop the 
   algorithm initiates a new outer loop.  If difsample<convergence in two contiguous outer loop the 
   algorithm stops. The convergence is achieved.}  
  }
%%   \item{\dots}{ not used. }
}

\references{Falorsi, P.D. and Righi P.(2015). Generalized framework for defining the optimal inclusion probabilities 
 of one-stage sampling designs for multivariate and multi-domain surveys, \emph{Survey Methodology}, 41, n.1, 215-236.
}

\author{Paolo Righi} 
\note{The instrumental CV thresholds in the errors data frame and hist_constraints matrix are based on the variance thresholds
 shown in formula (5.2) in Falorsi and Righi (2015). 
The  CV indices cumulate the combination of the driving variables by the domain of estimates.
When the CV combination specific thresholds are taken into account (see for example \link{errors.grad}), each index
corresponds to the index given in the data frame of CV thresholds.  
When the CV driving variable by partition specific thresholds are taken into account 
(see for example \link{errors.grad.matrix}) the indices are built automatically.
\bold{Example}: let us consider two driving variables and two domain partitions,
 the indices are applied according to the following order;  first driving variable (YPRED1)
by domains of the first partition (MARG1);
first driving variable (YPRED1) by domains of the second partition (MARG2);
 the second driving variable (YPRED2) by domains of the first partition (MARG1);
  second driving variable (YPRED2) by domains of the second partition (MARG2).
  
Dom variable in the errors  data frame is an instrumental variable. 
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{parameters}, \link{errors.grad}, \link{errors.grad.matrix}}. 
}

\examples{
\dontrun{
#The below examples work
####################################
## Example 1: graduates allocation
##            allocation in one step 
####################################
data(graduates)
data(errors.grad)
par<-parameters(graduates, "id_rek", "prof.size", "start.allocation", "dom", "y", "vary",
errors.grad, "cvt", vars = 2, nmarg = 2)
output_allocation<-allocation(par, output_mw, maxiter = 5, minouter=100, maxouter=107,
minprof=0, integ.solution="NO ", convergence = 0.01)

#Results
#Data frame with the inclusion probabilities
head(output_allocation$output)
#Statistics on the convergence process
head(output_allocation$hist_stat)
head(output_allocation$hist_expected_cv)
head(output_allocation$hist_constraints)


#####################################
## Example 2: graduates allocation
##            allocation in two steps 
#####################################


#First step - one outer loop iteration. Set the allocation function:  
output_mw1<-allocation(par, maxiter = 5, minouter=100, maxouter=100, convergence = 0.01)

#Save the partial output 
#setwd(".......")
save(output_mw1, file= "output_mw1.RData")

# If the second step allocation is performed in a new R session
#setwd("...")  
#load("...\\output_mw1.RData")       

#Second Step allocation
output_mwf<-allocation(par, output_mw1, maxiter = 5, 101, 110, convergence = 0.01)

#Results
#Data frame with the inclusion probabilities
head(output_mwf$output)
#Statistics on the convergence process
head(output_mwf$hist_stat)
head(output_mwf$hist_expected_cv)
head(output_mwf$hist_constraints)


##################################################
## Example 3: Allocation defined on two partitions
##            data frame ict.enterprises
##################################################
#Note the variables DOM1, DOM2 and DOM3 identify three partitions of the population. 
#Objective: allocate the sample taking into account DOM2 and DOM3. One solution is two
#delete DOM1 and rename DOM2 as DOM1 and DOM3 as DOM2.   
#Otherwise the user can act on the CV thresholds as follows:

data(ict.enterprises)
data(ict.errors_2marg)
par.ict<-parameters(ict.enterprises, "STRATO", "N", "initial.size", "DOM", "M", "V",
ict.errors_2marg, "CV", vars = 2, nmarg = 3)
ict.all<-allocation(par.ict, output_mw, maxiter = 5, minouter=100, maxouter=107,
minprof=0, integ.solution="NO ", convergence = 0.10)

#The use of CVs equal to 999 in the first row related to DOM1 excludes such partition in the
# definition of the allocation

}

}