\name{parameters}
\alias{parameters}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Definition of the parameters for the allocation procedure 
%%  ~~function to do ... ~~
}
\description{Defines the parameters for the allocation procedure. Imports the input data frames and sets the number of driving variables and domain partitions (margins)
%%  ~~ A concise (1-5 lines) description of what the function does. ~~
}
\usage{
parameters(namedata, id, pop, start_size, MARG, YPRED, RES, err, CV, vars = 2, 
           nmarg = 2)
}

\arguments{
  \item{namedata}{data frame with input values for the allocation procedure; its number of rows is the number of profiles}
  \item{id}{name of the variable identifying the profile}
  \item{pop}{name of the variable identifying the profile population size}
  \item{start_size}{name of the variable identifying the initial profile sample size; typically set equal to 0.5*pop}
  \item{MARG}{the name of the prefix specifying the domain partition (margin)}
  \item{YPRED}{the name of the prefix specifying the predicted values of the allocation driving variables}
  \item{RES}{the name of the prefix specifying the prediction  variance of the allocation driving variables}
  \item{err}{data frame with the coefficient of variation thresholds}
  \item{CV}{the name of the prefix specifying the the coefficient of variation thresholds in the err data frame}
  \item{vars}{an integer specifying the number of driving variables; by default, its value is 2}
  \item{nmarg}{an integer specifying the number of domain partitions (margins); by default, its value is 2}
}
\details{The records of the namedata data frame identify the profiles of the population. A profile is given by the values of the YPRED variables and MARG variables.
Population units with the same values of this set of variables define a unique record (profile) in the data frame. 
The function aims to store the metadata describing the sampling allocation and
the matrices required by the allocation function. 
Depending on the dimension of the allocation problem (number of profiles, 
number of driving variables and number of margins), the function could be time spending.

The metadata and the matrices are the input for the allocation function.
}
\value{The function returns a LIST with several objects  required to run the allocation function.
 \item{namedata}{the input data frame}
 \item{modelmatrix}{matrix (dimensions: number profiles - by - number of domains of interest) whose columns
 identify the domains of interest. Each column is the membership indicator variable 
(1 - record belongs to the domain, 0 - otherwise). The column are ordered as follows: ascending ordered domain partitions
 by ascending ordered domains in a given partition. Zero value occurs when the colomn identifies a domain not including the record.}
\item{matrixres }{matrix  (dimensions: number profiles - by - number of driving variables by number of domains of interest)
 whose columns have the prediction variances or 0 values. 
The column are ordered as follows:ascending ordered driving variables by ascending ordered domain partitions by ascending ordered domains in a given partition.
 Zero value occurs when the colomn identifies a domain not including the record.}
 
\item{cvt_err }{data frame (dimension: 1 - by - number of driving variables by number of domains of interest)
 whose columns have the coefficient of variation.}
 \item{meta}{LIST containing meta information for the allocation function. Most of the objects are described in the argument section (id, pop, start_size, 
 MARG,vars, nmarg). Furthrmore: dim_dom is the data frame including the variable MARG (number of domains for each partition) 
 and MARGT the overall number of domains; domvar is a numeric value counting the number od domains by the number of driving variables; fpc is 
 the finite population correction factor (Falorsi and Righi, 2015); POP is the profile population size; STRATA is the data frame required in 
 the modified Chromy algorithm (Falorsi and Righi, 2015).}
}

\references{Falorsi, P.D. and Righi P.(2015). Generalized framework for defining the optimal inclusion probabilities 
 of one-stage sampling designs for multivariate and multi-domain surveys, \emph{Survey Methodology}, 41, n.1, 215-236.
}

\author{
Paolo Righi
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{allocation}}
}

\examples{
\dontrun{
#The below examples work
#######################################################################
## Example 1: graduates with specific variable by domains CV thresholds
#######################################################################
# Set up the meta information for the graduates example. Data frame graduates contains 
#1679 records (profiles) for 3427 graduates.

data(graduates)
head(graduates)
data(errors.grad)
errors.grad
#Set up the function: two driving variables (vars=2) and two domain partitions (nmarg=2)
par<-parameters(graduates, "id_rek", "prof.size", "start.allocation", "dom", "y", "vary",
errors.grad, "cvt", vars = 2, nmarg = 2)

# Metadata stored in the par LIST containing the following objects
str(par)
# Input  data frame
head(par$namedata)
# Matrix modelmatrix 
dim(par$modelmatrix)
head (par$modelmatrix)
# Matrix matrixres
dim(par$matrixres)
head (par$matrixres)
#Data frame cvt_err
head(par$cvt_err)
# List including meta parameters
str(par$meta)

##########################################################################
## Example 2: graduates with driving variables by partitions CV thresholds
##########################################################################
#Note the CV thresholds are the same of example 1 but they arranged in a different block way. 
data(graduates)
head(graduates)
data(errors.grad.matrix)
#Set up the function: two driving variables (vars=2) and two domain partitions (nmarg=2)
par<-parameters(graduates, "id_rek", "prof.size", "start.allocation", "dom", "y", "vary", 
errors.grad.matrix, "cv_y", vars = 2, nmarg = 2)

# Metadata stored in the par LIST containing the following objects
str(par)
# Input  data frame
head(par$namedata)
# Matrix modelmatrix 
dim(par$modelmatrix)
head (par$modelmatrix)
# Matrix matrixres
dim(par$matrixres)
head (par$matrixres)
#Data frame cvt_err. The order of the CVs follows the rule given in Note of allocation function. 
#The order is the same of the Example 1. 
head(par$cvt_err)
# List including meta parameters
str(par$meta)
}
}